"""
Word document processor implementation.
"""
import io
import os
from datetime import datetime
from typing import Dict, Optional

from docx import Document as DocxDocument

from ..models.document import Document, DocumentMetadata, DocumentType
from .base import BaseDocumentProcessor


class DocxProcessor(BaseDocumentProcessor):
    """Processor for Word documents."""
    
    def load_document(self, file_path: str) -> Document:
        """Load a Word document from a file.
        
        Args:
            file_path: Path to the Word file
            
        Returns:
            Document: Loaded document
        """
        with open(file_path, "rb") as file:
            content = file.read()
            text = self.extract_text(content)
            metadata = self.get_metadata(file_path, content)
            chunks = self.create_chunks(text)
            
            return Document(
                content=text,
                metadata=metadata,
                chunks=chunks
            )
    
    def extract_text(self, content: bytes) -> str:
        """Extract text content from Word document bytes.
        
        Args:
            content: Word file content as bytes
            
        Returns:
            str: Extracted text content
        """
        doc = DocxDocument(io.BytesIO(content))
        
        text_content = []
        for paragraph in doc.paragraphs:
            if paragraph.text.strip():
                text_content.append(paragraph.text)
        
        # Extract text from tables
        for table in doc.tables:
            for row in table.rows:
                for cell in row.cells:
                    if cell.text.strip():
                        text_content.append(cell.text)
        
        return "\n".join(text_content)
    
    def get_metadata(self, file_path: str, content: bytes) -> DocumentMetadata:
        """Extract metadata from Word document.
        
        Args:
            file_path: Path to the Word file
            content: Word file content as bytes
            
        Returns:
            DocumentMetadata: Extracted metadata
        """
        doc = DocxDocument(io.BytesIO(content))
        
        # Extract core properties
        core_props = doc.core_properties
        custom_metadata: Dict[str, str] = {}
        
        if core_props:
            if core_props.author:
                custom_metadata["author"] = core_props.author
            if core_props.title:
                custom_metadata["title"] = core_props.title
            if core_props.created:
                custom_metadata["created"] = str(core_props.created)
            if core_props.modified:
                custom_metadata["modified"] = str(core_props.modified)
            if core_props.last_modified_by:
                custom_metadata["last_modified_by"] = core_props.last_modified_by
            if core_props.revision:
                custom_metadata["revision"] = str(core_props.revision)
        
        return DocumentMetadata(
            filename=os.path.basename(file_path),
            file_type=DocumentType.DOCX,
            file_size=len(content),
            num_pages=len(doc.paragraphs),  # Approssimazione
            author=custom_metadata.get("author"),
            title=custom_metadata.get("title"),
            custom_metadata=custom_metadata,
        ) 